##
## SNMPTRAPD perl initialization file.
##

# DO NOT EDIT THIS FILE
#
# This file is intended for initialization of the perl subsystem used by
# snmptrapd.  Though it could be modified by a local administrator to do
# things, it should not be as it will be overwritten by future
# Net-SNMP installations.  Run "perldoc NetSNMP::TrapReceiver" for
# documentation on how to use perl within snmptrapd.

use NetSNMP::TrapReceiver;


